@@ -5,11 +5,13 @@ Class definition
 import json
 
 import pandas
+import pandas as pd
 import requests
 import urllib3
 from pandas import json_normalize
 from nexus_api.NexusRequest import NexusRequest
 from nexus_api.NexusValue import NexusValue
+from nexus_api.exceptions import VoidDataframeException, CorruptDataframeException
 
 
 def WarningsAndJson(func):
@@ -188,7 +190,7 @@ class APINexus:
         if not variable_name in variables_names:
             print("Se procede a crear la variable con nombre "
                   + variable_name
-                  + "Se devuelve json con uid y nombre de variable creada"
+                  + ". Se devuelve json con uid y nombre de variable creada"
                   )
             payload = '["' + variable_name + '"]'
             print(payload)
@@ -222,9 +224,6 @@ class APINexus:
             variable_uid = list(
                 variables_norm[variables_norm.name == variable_name].uid
             )[0]
-            print(
-                "El uid de la variable que se ha solicitado escribir es " + variable_uid
-            )
             urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
             url_completa = self.url_NX + "/api/Tags/realtime/insert"
             nexusvalue = NexusValue(variable_uid, variable_value)
@@ -441,7 +440,7 @@ class APINexus:
 
         return dataReceived
 
-    def filter_installation(self, Datefrom, Dateto, columnas, filter_txt=None, resolucion=3, fuente=0):
+    def filter_installation(self, Datefrom, Dateto, columnas, filter_txt=None, uids=None, resolucion=3, fuente=0) -> pd.DataFrame:
         """
         la funcion recibe como parametros la fecha ini, fecha fin, un df con los uid y
         los nombres de las variables de la instalación y el filtro de texto aplicar
@@ -450,45 +449,43 @@ class APINexus:
             Datefrom (datetime)
             Dateto (datetime)
             columnas (dataframe)
-            filter_txt (optonal: list or str)
+            uids: (optional) (list): lista de los uids de las variables deseadas. No puede usarse con filter_txt
+            filter_txt (optional: list or str): lista de nombres de las variables deseadas. No puede usarse con uids
             resolucion (int): de 0 a 10 [ RES_30_SEC, RES_1_MIN, RES_15_MIN, RES_1_HOUR, RES_1_DAY, RES_1_MONTH, RES_1_YEAR, RES_5_MIN, RES_200_MIL, RES_500_MIL, RES_1_SEC ]
             fuente (int): [0 -->RAW, 1 -->STATS_PER_HOUR, 2 -->STATS_PER_DAY, 3 -->STATS_PER_MONTH, 4 -->TRANSIENT]
         Returns:
             filtered_hist (dataframe)
         """
-        uids = []
-        if isinstance(filter_txt, list):
-            for filter in filter_txt:
-                uids_loop = list(columnas[columnas['name'].str.contains(filter, case=False)].uid)
-                uids.extend(uids_loop)
-        else:
-            if filter_txt:
-                uids = list(columnas[columnas['name'].str.contains(filter_txt, case=False)].uid)
-        if not uids:
-            print('Los filtros proporcionados no encuentran ninguna variable. Se devolverá toda la instalación')
-            uids = list(columnas.uid)
+        # Check parameters
+        if uids is not None and filter_txt is not None:
+            raise ValueError('Error en uids y filter_txt. No pueden usarse ambos parámetros de búsqueda')
+        if uids is None:
+            uids = []
+            if isinstance(filter_txt, list):
+                for filter in filter_txt:
+                    uids_loop = list(columnas[columnas['name'].str.contains(filter, case=False)].uid)
+                    uids.extend(uids_loop)
+            else:
+                if filter_txt:
+                    uids = list(columnas[columnas['name'].str.contains(filter_txt, case=False)].uid)
+            if not uids:
+                print('Los filtros proporcionados no encuentran ninguna variable. Se devolverá toda la instalación')
+                uids = list(columnas.uid)
+        # Remove duplicate UIDS
+        uids = list(set(uids))
         fecha_ini = Datefrom
         fecha_fin = Dateto
         nexusRequest = NexusRequest(uids, fecha_ini, fecha_fin, fuente, resolucion)
         filtered_hist = self.callGetTagsHistory(nexusRequest)
         filtered_hist = json_normalize(filtered_hist)
-
-        # Check that dataframe is not empty
-        if filtered_hist.empty:
-            raise VoidDataframeException
-        # Check content
-        if not {'timeStamp', 'uid', 'value'}.issubset(filtered_hist.columns):
-            raise CorruptDataframeException(filtered_hist)
-
+        # Check that there is no error in the dataframe
+        self.__check_response_error_df(filtered_hist)
         filtered_hist.timeStamp = pandas.to_datetime(filtered_hist.timeStamp, unit='s')
         diccio = dict([(i, j) for i, j in zip(columnas.uid, columnas.name)])
         filtered_hist['name'] = filtered_hist['uid'].map(diccio)
-        filtered_hist = filtered_hist.rename(columns={'uid': 'name'})
-        filtered_hist.set_index(filtered_hist.timeStamp, inplace=True)
-        del filtered_hist['timeStamp']
         return filtered_hist
 
-    def filter_tagview(self, Datefrom, Dateto, columnas, uid_vista, filter_txt=None, resolucion=3, fuente=0):
+    def filter_tagview(self, Datefrom, Dateto, columnas, uid_vista, filter_txt=None, uids=None, resolucion=3, fuente=0) -> pd.DataFrame:
         """
         La funcion recibe como parametros la fecha ini, fecha fin, un df con los uid y
         los nombres de las variables de la instalación y el filtro de texto aplicar [pueden ser varios]
@@ -496,39 +493,40 @@ class APINexus:
             Datefrom (datetime)
             Dateto (datetime)
             columnas (dataframe)
-            filter_txt (optional: list or str)
+            uids: (optional) (list): lista de los uids de las variables deseadas. No puede usarse con filter_txt
+            filter_txt (optional: list or str): lista de nombres de las variables deseadas. No puede usarse con uids
             resolucion (int): de 0 a 10 [ RES_30_SEC, RES_1_MIN, RES_15_MIN, RES_1_HOUR, RES_1_DAY, RES_1_MONTH, RES_1_YEAR, RES_5_MIN, RES_200_MIL, RES_500_MIL, RES_1_SEC ]
             fuente (int): [0 -->RAW, 1 -->STATS_PER_HOUR, 2 -->STATS_PER_DAY, 3 -->STATS_PER_MONTH, 4 -->TRANSIENT]
         Returns:
             filtered_hist (dataframe)
         """
-        uids = []
-        if isinstance(filter_txt, list):
-            for filter in filter_txt:
-                uids_loop = list(columnas[columnas['name'].str.contains(filter, case=False)].uid)
-                uids.extend(uids_loop)
-        else:
-            if filter_txt:
-                uids = list(columnas[columnas['name'].str.contains(filter_txt, case=False)].uid)
-        if not uids:
-            print('Los filtros proporcionados no encuentran ninguna variable. Se devolverá toda la instalación')
-            uids = list(columnas.uid)
+        # Check parameters
+        if uids is not None and filter_txt is not None:
+            raise ValueError('Error en uids y filter_txt. No pueden usarse ambos parámetros de búsqueda')
+        if uids is None:
+            uids = []
+            if isinstance(filter_txt, list):
+                for filter in filter_txt:
+                    uids_loop = list(columnas[columnas['name'].str.contains(filter, case=False)].uid)
+                    uids.extend(uids_loop)
+            else:
+                if filter_txt:
+                    uids = list(columnas[columnas['name'].str.contains(filter_txt, case=False)].uid)
+            if not uids:
+                print('Los filtros proporcionados no encuentran ninguna variable. Se devolverá toda la instalación')
+                uids = list(columnas.uid)
+        # Remove duplicate UIDS
+        uids = list(set(uids))
         fecha_ini = Datefrom
         fecha_fin = Dateto
         # fuente: [0 -->RAW, 1 -->STATS_PER_HOUR, 2 -->STATS_PER_DAY, 3 -->STATS_PER_MONTH, 4 -->TRANSIENT]
         # resolucion: de 0 a 10 [ RES_30_SEC, RES_1_MIN, RES_15_MIN, RES_1_HOUR, RES_1_DAY, RES_1_MONTH, RES_1_YEAR,
         # RES_5_MIN, RES_200_MIL, RES_500_MIL, RES_1_SEC ]
-
         nexusRequest = NexusRequest(uids, fecha_ini, fecha_fin, fuente, resolucion)
         filtered_hist = self.callGetDataviewHistory(uid_vista, nexusRequest)
         filtered_hist = json_normalize(filtered_hist)
-
-        # Check that dataframe is not empty
-        if filtered_hist.empty:
-            raise VoidDataframeException
-        # Check content
-        if not {'timeStamp', 'uid', 'value'}.issubset(filtered_hist.columns):
-            raise CorruptDataframeException(filtered_hist)
+        # Check dataframe for errors
+        self.__check_response_error_df(filtered_hist)
         filtered_hist.timeStamp = pandas.to_datetime(filtered_hist.timeStamp, unit='s')
         diccio = dict([(i, j) for i, j in zip(columnas.uid, columnas.name)])
         filtered_hist['name'] = filtered_hist['uid'].map(diccio)
@@ -548,17 +546,21 @@ class APINexus:
             uids.extend(alarms['uid'][alarms['group'].str.contains(group, case=False)].to_list())
         return uids
 
-
-class VoidDataframeException(Exception):
-    def __init__(self, message='dataframe is empty'):
-        self.message = message
-        # Show full traceback
-        super().__init__(self.message)
-
-
-class CorruptDataframeException(Exception):
-    def __init__(self, df, message='dataframe does not match Nexus structure'):
-        self.message = f'{message} - {df.head()}'
-        self.df = df
-        # Show full traceback
-        super().__init__(self.message)
+    def __check_response_error_df(self, filtered_hist: pd.DataFrame) -> None:
+        """
+        Checks the received dataframe from API request and raises an error if not valid
+        """
+        # Check that there is no error in the dataframe
+        if 'StatusCode' in filtered_hist.keys():
+            message = filtered_hist['Message']
+            code = filtered_hist['StatusCode']
+            ex_message = f'Status Code: {code}: {message}'
+            raise ValueError(ex_message)
+        # Check that dataframe is not empty
+        elif filtered_hist.empty:
+            raise VoidDataframeException
+        # Check content
+        elif not {'timeStamp', 'uid', 'value'}.issubset(filtered_hist.columns):
+            raise CorruptDataframeException(filtered_hist)
+        else:
+            pass
